
%if 0

Resident code of lClock
 2021 by C. Masloch

Usage of the works is permitted provided that this
instrument is retained with the works, so that any entity
that uses the works is notified of this instrument.

DISCLAIMER: THE WORKS ARE WITHOUT WARRANTY.

%endif

%if 0

Supported Int2D functions:

AMIS - Installation check
INP:	al = 00h
OUT:	al = 0FFh
	cx = Private version number (currently 0100h)
	dx:di-> signature: "ecm     ","lClock  "

AMIS - Get private entry point - NOP: no private entry point
INP:	al = 01h
OUT:	al = 00h

AMIS - Uninstall - NOP: no resident uninstaller or NOP: can't uninstall
INP:	al = 02h
OUT:	If installed from command line,
	 al = 03h
	 bx = memory block of resident TSR (cs)
	(If internal (pre-installed),
	 al = 01h
	If with resident uninstaller,
	 al = 01h if unsuccessful
	 al = 0FFh if successful)

AMIS - Request pop-up - NOP: no pop-up
INP:	al = 03h
OUT:	al = 00h

AMIS - Determine chained interrupts
INP:	al = 04h
OUT:	al = 04h
	dx:bx-> interrupt hook list (Int2D always.)

AMIS - Get hotkeys - NOP: no hotkeys
INP:	al = 05h
OUT:	al = 00h

AMIS - Get device driver information - NOP: no device
INP:	al = 06h
OUT:	al = 00h

AMIS - Reserved for AMIS
INP:	al = 07h..0Fh
OUT:	al = 00h

TSR - Get compatible version
INP:	al = 10h
OUT:	al = 0FFh
	cx = compatible version number (currently 0100h)

TSR - Get ctrl0
INP:	al = 20h
OUT:	al = size of returned control data (not 0)
	dx:bx-> control data
	Unsupported bits switched off

TSR - Get ctrl1
INP:	al = 21h
OUT:	al = size of returned control data (not 0)
	dx:bx -> control data

TSR - Reserved for TSR
INP:	al = 11h..1Fh, 22h..FFh
OUT:	al = 00h

%endif

%if TSR_VERP != ver(1,00)		; if defined one doesn't match current
 %error "TSR version doesn't match"
%endif

	numdef BLANK_PREFIX, 1
	numdef BLANK_LOW_HOUR, 0
	numdef 12HOUR, 0
	numdef AMPM, 0, 2		; 1 = single letter, 2 = two letters
	numdef AMPM_CAPITAL, 0
	numdef AMPM_NOON_IS_AM, 0
	numdef SECONDS, 1
	numdef WRITE_UPDATE_ONLY, 0
	numdef DIVISOR_HOURS, 1_0007h
	numdef DIVISOR_HOURS_MORE_ACCURATE, 1
	numdef DIVISOR_HOURS_TIMES_1000, 65543_333
	numdef DIVISOR_MINUTES, 444h
	numdef DIVISOR_MINUTES_MORE_ACCURATE, 1
	numdef DIVISOR_MINUTES_TIMES_10, 1092_4
	numdef DIVISOR_SECONDS, 12h
	numdef DIVISOR_SECONDS_MORE_ACCURATE, 1
	numdef DIVISOR_SECONDS_TIMES_1000, 18_206

%define VARIANT
%if _DIVISOR_HOURS_MORE_ACCURATE
 %xdefine VARIANT VARIANT +HOURS_ACC
%endif
%if _DIVISOR_MINUTES_MORE_ACCURATE
 %xdefine VARIANT VARIANT +MINUTES_ACC
%endif
%if _DIVISOR_SECONDS_MORE_ACCURATE
 %xdefine VARIANT VARIANT +SECONDS_ACC
%endif
%ifempty VARIANT
 %define VARIANT none
%endif
%warning Building variant: VARIANT


handler:
				; AMIS stuff
amissig:
.ven:	fill 8,32,db "ecm"		; vendor
.prod:	fill 8,32,db "lClock"		; product
%if 0
.desc:	asciz ;"Clock display TSR"	; save memory by omitting the description
%else
.desc:	asciz "Clock display TSR"	; description
%endif
%if $ - .desc > 64
 %error AMIS description too long
%endif

amisintr:
.i1C:	db 1Ch
	dw i1C
.i2D:	db 2Dh
	dw i2D

	align 4
				; TSR data
ctrl1:	dw 80 * 2, 0B800h		; -> behind where to write to
ctrl0:	db 1000_0000b			; control byte
					; bit 6: state 1
					;     7: state 2
%if _WRITE_UPDATE_ONLY
second:	db 255
%endif
buffer:
%if _BLANK_PREFIX
	db 32
%endif
	times 2 + 1 + 2 + 3 * !!_SECONDS + _AMPM db 0


i2D.uninstall:
	inc ax				; (= 03h) safe to remove but no resident uninstaller
	mov bx, cs			; = segment
i2D.hwreset equ $-1		; (second byte of mov bx, cs is same as the retf opcode)
	iret

	align 2
iispentry i2D, 0, i2D
	cmp ah, 0
amisnum equ $-1				; AMIS multiplex number (data for cmp opcode)
	je .handle			; our multiplex number -->
	jmp far [cs:.next]		; else go to next handler -->

.handle:
	test al, al
	jz .installationcheck		; installation check -->
	cmp al, 02h
	je .uninstall			; uninstallation -->
	cmp al, 04h
	je .determineinterrupts		; determine hooked interrupts -->
	cmp al, 10h
	je .getver			; get compatible version -->
	cmp al, 20h
	je .getctrl0			; get ctrl0 -->
	cmp al, 21h
	je .getctrl1			; get ctrl1 -->
				; all other functions are reserved or not supported by TSR
.nop:
	mov al, 0			; show not implemented
	iret

.installationcheck:
	dec al				; (= FFh) show we're here
	mov cx, TSR_VERP		; = version
	xor di, di			; dx:di -> AMIS signature strings of this program
.iret_dx_cs:
	mov dx, cs
.iret:
	iret

.determineinterrupts:			; al = 04h, always returns list
	mov bx, amisintr		; dx:bx -> hooked interrupts list
	jmp short .iret_dx_cs

.getver:
	mov al, 0FFh			; show call supported
	mov cx, TSR_VERC		; = compatible version
	iret

.getctrl0:
	mov bx, ctrl0			; dx:bx -> ctrl0
	mov al, 1			; size of ctrl0: 1 byte
	and byte [cs:bx], _state2
%if _WRITE_UPDATE_ONLY
	mov byte [cs:second], 255
%endif
	jmp short .iret_dx_cs

.getctrl1:
	mov bx, ctrl1			; dx:bx -> ctrl1
	mov al, 4			; size of ctrl1: 4 byte
	jmp short .iret_dx_cs


	align 2
iispentry i1C, 0, i2D
	test byte [cs:ctrl0], _state2
	jz .chain
	push ax
	push cx
	push dx
	push bx
	push ds
	push es
	push si
	push di
	push bp

	cld
	xor ax, ax
	mov ds, ax
	mov dx, word [46Ch]
	mov bx, word [46Ch + 2]	; bx:dx = dividend (0..18_00AFh)
.demo_entry:
%if _DIVISOR_HOURS_MORE_ACCURATE
	mov si, dx
	mov ax, 1000
	 push ax
	mul bx
	xchg ax, bx		; bx = high word 1
	 pop ax			; = 1000
	mul si			; dx:ax = high word 2:low word
	add dx, bx
	xchg bx, ax		; dx:bx = times 1000
	xchg dx, bx		; bx:dx
	mov bp, _DIVISOR_HOURS_TIMES_1000 >> 16
	mov ax,	_DIVISOR_HOURS_TIMES_1000 & 0FFFFh
%else
	mov bp, _DIVISOR_HOURS >> 16
	mov ax,	_DIVISOR_HOURS & 0FFFFh
				; bp:ax = 1_0007h (ticks per hour)
%endif

	xor si, si
	xor di, di		; clear variable si:di

	mov cx, 32
.bitloop:
	shl dx, 1
	rcl bx, 1
	rcl di, 1
	rcl si, 1		; si:di:bx:dx << 1
	cmp si, bp		; does the divisor fit into si:di here ?
	jne .trynext_if_B
	cmp di, ax
.trynext_if_B:
	jb .trynext		; no -->

.goesinto:
	sub di, ax
	sbb si, bp		; subtract divisor
	inc dx			; set a bit of the result (bit was zero before, never carries)
.trynext:
	loop .bitloop		; loop for 32 bits

%if _DIVISOR_HOURS_MORE_ACCURATE
	push dx
	xor dx, dx
	xchg ax, si
 %if _DIVISOR_MINUTES_MORE_ACCURATE
	mov cx, 100
 %else
	mov cx, 1000
 %endif
	div cx			; dx = remainder, ax = high word
	xchg ax, di		; di = high word, dx:ax = remainder:low word
	div cx			; dx = remainder, ax = low word
	xchg ax, si		; si = low word
	xchg di, si		; si:di = divided
	pop dx
%endif

		; bx:dx = quotient (hour, 0..23)
		; si:di = remainder (ticks within hour, 0..1_0006h)
		; (remainder times 10 if HOURS + MINUTES more accurate)
	 push cs
	 pop es
	 push cs
	 pop ds
	push di
	mov al, dl
%if _AMPM
 %if _AMPM == 1
  %if _AMPM_CAPITAL
	mov bl, "A"
  %else
	mov bl, "a"
  %endif
 %else
  %if _AMPM_CAPITAL
	mov bx, "AM"
  %else
	mov bx, "am"
  %endif
 %endif
 %if _AMPM_NOON_IS_AM
	test al, al
	jz .pm
 %endif
	cmp al, 12
 %if _AMPM_NOON_IS_AM
	jbe @F
 %else
	jb @F
 %endif
.pm:
 %if _AMPM_CAPITAL
	mov bl, "P"
 %else
	mov bl, "p"
 %endif
@@:
%endif
%if _12HOUR
	cmp al, 12
	jbe @F
	sub al, 12
@@:
	test al, al
	jnz @F
	mov al, 12
@@:
%endif
	mov di, buffer + !!_BLANK_PREFIX
				; -> buffer
	call sto_time
%if _BLANK_LOW_HOUR
	cmp byte [di - 2], '0'
	jne @F
	mov byte [di - 2], 32
@@:
%endif
	mov al, ':'
	stosb
	pop ax			; si:ax = ticks within hour
%if _DIVISOR_MINUTES_MORE_ACCURATE
 %ifn _DIVISOR_HOURS_MORE_ACCURATE
	mov cx, 10
	mul cx			; = high word 2:low word
	push ax
	xchg bp, dx		; bp = high word 2
	mov ax, si
	mul cx
	add bp, ax
	pop ax			; bp:ax = times 10
	mov dx, bp
 %else
	mov dx, si
 %endif
	mov cx, _DIVISOR_MINUTES_TIMES_10
	div cx			; ax = quotient
				; dx = remainder * 10
 %ifn _DIVISOR_SECONDS_MORE_ACCURATE
	push ax
	mov cx, 10
	xchg ax, dx		; ax = remainder * 10
	xor dx, dx		; dx:ax = remainder * 10
	div cx			; ax = remainder
	xchg ax, dx		; dx = remainder
	pop ax
 %endif
%else
	mov dx, si
	mov cx, _DIVISOR_MINUTES
				; ticks per minute
	div cx
%endif
		; ax = quotient (minute, 0..59)
		; dx = remainder (ticks within minute, 0..443h)
		; (remainder times 10 if MINUTES + SECONDS more accurate)
	call sto_time
%if _SECONDS || _WRITE_UPDATE_ONLY
 %if _SECONDS
	mov al, ':'
	stosb
 %endif
	mov ax, dx
 %if _DIVISOR_SECONDS_MORE_ACCURATE
  %if _DIVISOR_MINUTES_MORE_ACCURATE
	mov cx, 100
  %else
	mov cx, 1000
  %endif
	mul cx
	mov cx, _DIVISOR_SECONDS_TIMES_1000
	div cx
 %else
	mov cl, _DIVISOR_SECONDS
	div cl
 %endif
		; al = quotient (second, 0..59)
 %if _WRITE_UPDATE_ONLY
	cmp al, byte [second]
	je .skip
	mov byte [second], al
 %endif
 %if _SECONDS
	call sto_time
 %endif
%endif

%if _AMPM
	xchg ax, bx
 %if _AMPM == 1
	stosb
 %else
	stosw
 %endif
%endif

	mov si, buffer		; -> written time
	mov cx, di
	sub cx, si		; = length of written time
.demo_retf:
	les di, [ctrl1]
	sub di, cx
	sub di, cx		; -> destination

@@:
	movsb
	inc di
	loop @B

.skip:
	pop bp
	pop di
	pop si
	pop es
	pop ds
	pop bx
	pop dx
	pop cx
	pop ax
.chain:
	jmp far [cs:.next]


sto_time:
	mov cl, 10
	xor ah, ah
	div cl
	add ax, "00"
	stosw
	retn


	align 16
	endarea handler
